use crate::eval::{Context, EvalError, Value};

pub fn prelude(context: &mut Context) {
	context.define_built("push", |args, _ctx| {
		let mut args = args.into_iter();
		let list = args.next().ok_or(EvalError::NotFound("$1".into()))?;
		let Value::Array(list) = list else {
			return Err(EvalError::WrongType);
		};
		list.borrow_mut().extend(args);
		Ok(Value::None)
	});

	context.define_built("pop", |args, _ctx| {
		let list = args.first().ok_or(EvalError::NotFound("$1".into()))?;
		let Value::Array(list) = list else {
			return Err(EvalError::WrongType);
		};
		Ok(list.borrow_mut().pop().unwrap_or(Value::None))
	});

	context.define_built("len", |args, _ctx| {
		let list = args.first().ok_or(EvalError::NotFound("$1".into()))?;
		let Value::Array(list) = list else {
			return Err(EvalError::WrongType);
		};
		Ok(Value::Num(list.borrow().len() as f64))
	});

	context.define_built("str", |args, _ctx| {
		let first = args.first().ok_or(EvalError::NotFound("$1".into()))?;
		Ok(Value::Str(first.serialize()))
	});
}
